#Simulate some noisy data
simulated = 10*sin(5*pi*(1:1000)/1000) + rnorm(1000,0,3)

ma = function(x,k=1) filter(x,rep(1/k,k),sides=1)
ema = function(x,a=0.5,val=0) {
	sapply(x,function(j) {
		val <<- (1-a)*val + a*j
		val
	})
}

#Figure 11-1
plot.new()
par(mfrow=c(3,1))
for(i in c(10,20,30)) {
	plot(simulated,pch=20,cex=0.5,col="lightgray",xlab="Time",ylab="Value")
	lines(ma(simulated,i))
	title(paste("MA(",i,")",sep=""))
}
#Figure 11-2
plot.new()
par(mfrow=c(3,1))
for(i in c(10,20,30)) {
	plot(simulated,pch=20,cex=0.5,col="lightgray",xlab="Time",ylab="Value")
	lines(ma(simulated,i),lty=2)
	lines(ema(simulated,2/(i+1)))
	title(paste("EMA(",i,")",sep=""))
}

#Figure 11-3
plot.new()
par(mfrow=c(1,3))
curve(2/(1+exp(-x)) - 1,-6,6,main=quote(1/(1+e^-phi)),ylab="Output",xlab="Input")
curve(tanh,-6,6,main=quote(tanh(phi)),ylab="Output",xlab="Input")
curve(ifelse(x < 0,-1,1),-6,6,main="Heaviside Step",ylab="Output",xlab="Input")


#Difference audience visit rates.
aud.1.rate = 1000*(sin(2*pi*(0:23)/23) + 1.5)
aud.2.rate = 1000*(sin(2*pi*(0:23)/23 + pi/2) + 1.5)

#Time based action rates
aud.1.ar = 0.10 + (sin(pi*(0:23)/23 + pi) + 1)/10
aud.2.ar = 0.01 + (sin(pi*(0:23)/23) + 1)/10

#Generate a dataset
resp = data.frame(
  hr=rep(0:23,30),
  aud1=rpois(30*24,aud.1.rate) + round(rnorm(30*24,0,1000)),
  aud2=rpois(30*24,aud.2.rate) + round(rnorm(30*24,0,500)))
resp$aud1.a = rbinom(30*24,resp$aud1,aud.1.ar)
resp$aud2.a = rbinom(30*24,resp$aud2,aud.2.ar)
resp$visits = resp$aud1+resp$aud2
resp$actions= resp$aud1.a+resp$aud2.a
boxplot(resp$visits ~ resp$hr)



x = read.delim(pipe("pbpaste","r"),sep="\t",header=TRUE)
plot(x[,2],pch=20,cex=0.5,col="lightgrey",main="Linear Regression Time-Series",ylab="",xlab="Time")
lines(x[,3],lty=2)
lines(x[,1],lwd=2)