package wiley.streaming.zookeeper;

import java.io.IOException;
import java.util.Collections;
import java.util.List;

import org.apache.zookeeper.AsyncCallback.ChildrenCallback;
import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.KeeperException;
import org.apache.zookeeper.WatchedEvent;
import org.apache.zookeeper.Watcher;
import org.apache.zookeeper.ZooDefs;
import org.apache.zookeeper.ZooKeeper;
import org.apache.zookeeper.data.Stat;

public class LeaderElection implements Watcher, ChildrenCallback {
	String    path;
	ZooKeeper client;
	String    node;
	public Integer   lock      = new Integer(0);
	public boolean   leader    = false;
	public boolean   connected = false;
	
	public LeaderElection(String connect,String path) throws IOException {
		this.path = path;
		client = new ZooKeeper(connect, 3000, this);
	}
	
	
	public void process(WatchedEvent event) {
		if(event.getType() == Watcher.Event.EventType.None) {
			switch(event.getState()) {
			case SyncConnected:
				connected = true;
				try {
					requestLeadership();
				} catch(Exception e) {
					//Quit
					synchronized(lock) {
						leader = false;
						lock.notify();
					}
				}
				break;
			case Expired:
			case Disconnected:
				synchronized(lock) {
					connected = false;
					leader    = false;
					lock.notify();
				}
				break;
			default:
				break;
			}
		} else {
			if(path.equals(event.getPath())) 
				client.getChildren(path, true, this, null);
		}
		
	}
	
	
	public void requestLeadership() throws Exception {
		Stat stat = client.exists(path, false);
		if(stat == null) {
			try {
				client.create(path, new byte[0], ZooDefs.Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
			} catch(KeeperException.NodeExistsException e) { }
		}
		client.getChildren(path, true, this, null);
		node = client.create(path+"/n_", new byte[0], ZooDefs.Ids.OPEN_ACL_UNSAFE, CreateMode.EPHEMERAL_SEQUENTIAL);
		System.out.println("My node is: "+node);
	}

	protected boolean isLeader(List<String> children) {
		if(children.size() == 0) return false;
		Collections.sort(children);
		System.out.println(path+"/"+children.get(0)+" should become leader. I am "+node);
		return (node.equals(path+"/"+children.get(0))); 
	}
	
	
	public void processResult(int rc, String path, Object ctx,
			List<String> children) {
		synchronized(lock) {
			leader = isLeader(children);
			System.out.println(node+" leader? "+leader);
			lock.notify();
		}
	}
	
	public void close() throws InterruptedException {
		client.close();
	}

	public boolean isLeader() {
		if(leader && connected) {
			System.out.println(node+" is the leader.");
			return leader;
		}
		System.out.println("Waiting for a change in the lock.");
		synchronized(lock) {
			try {
				lock.wait();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		return leader && connected;
	}
	

}
