package wiley.streaming.curator;

import static org.junit.Assert.*;

import org.apache.curator.framework.CuratorFramework;
import org.apache.curator.framework.CuratorFrameworkFactory;
import org.apache.curator.framework.api.BackgroundCallback;
import org.apache.curator.framework.api.CuratorEvent;
import org.apache.curator.framework.state.ConnectionState;
import org.apache.curator.framework.state.ConnectionStateListener;
import org.apache.curator.retry.ExponentialBackoffRetry;
import org.apache.zookeeper.data.Stat;
import org.junit.Before;
import org.junit.Test;

public class DataTest {

	CuratorFramework client;
	
	@Before
	public void setUp() throws Exception {
		client =
				  CuratorFrameworkFactory.builder()
				   .connectString("localhost:2181")
				   .retryPolicy(new ExponentialBackoffRetry(1000,3))
				   .namespace("subspace")
				   .build();
				client.getConnectionStateListenable().addListener(new ConnectionStateListener() {

					public void stateChanged(CuratorFramework arg0, ConnectionState arg1) {
						System.out.println("State Changed: "+arg1);
					}
					
				});
				client.start();
				Stat stat;
				if((stat = client.checkExists().forPath("test")) != null)
					client.delete()
					.deletingChildrenIfNeeded()
					.withVersion(stat.getVersion())
					.forPath("test");
	}

	@Test
	public void test() throws Exception {
		client.create().creatingParentsIfNeeded().forPath("test/string");		
		client.setData().forPath("test/string", "A Test String".getBytes());
		
		//Immediate (blocking) version
		System.out.println(new String(client.getData().forPath("test/string")));

		//Background (non-blocking) version
		client.getData().inBackground(new BackgroundCallback() {

			public void processResult(CuratorFramework arg0, CuratorEvent arg1)
					throws Exception {
				System.out.println(new String(arg1.getData()));
			}
		
		}).forPath("test/string");
	}

}
