package wiley.streaming.curator;

import static org.junit.Assert.*;

import java.util.List;

import org.apache.curator.framework.CuratorFramework;
import org.apache.curator.framework.CuratorFrameworkFactory;
import org.apache.curator.framework.state.ConnectionState;
import org.apache.curator.framework.state.ConnectionStateListener;
import org.apache.curator.retry.ExponentialBackoffRetry;
import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.data.Stat;
import org.junit.Before;
import org.junit.Test;

public class QueueTest {

	CuratorFramework client;
	
	@Before
	public void setUp() throws Exception {
		client =
				  CuratorFrameworkFactory.builder()
				   .connectString("localhost:2181")
				   .retryPolicy(new ExponentialBackoffRetry(1000,3))
				   .namespace("subspace")
				   .build();
				client.getConnectionStateListenable().addListener(new ConnectionStateListener() {

					public void stateChanged(CuratorFramework arg0, ConnectionState arg1) {
						System.out.println("State Changed: "+arg1);
					}
					
				});
		client.start();
		
		//Remove an existing state so that the test is idempotent
		if(client.checkExists().forPath("/queue") != null)
			client.delete().deletingChildrenIfNeeded().forPath("/queue");		
	}
	
	@Test
	public void test() throws Exception {
		if(client.checkExists().forPath("/queue") == null)
			client.create().forPath("/queue");
		
		for(int i=0;i<10;i++) {
			client.create().withMode(CreateMode.PERSISTENT_SEQUENTIAL).forPath("/queue/job-");			
		}
		
		for(String job : client.getChildren().forPath("/queue")) {
			System.out.println("Job: "+job);
		}
		
		System.out.println("\n\nStat Example");
		Stat stat = new Stat();
		List<String> jobs = client.getChildren()
		.storingStatIn(stat)
		.forPath("/queue");
		for(String job : jobs)
			System.out.println("Job: "+job);
		System.out.println("Number of children: "+stat.getNumChildren());
		
	}

}
