package wiley.realtime;

import cern.jet.math.Arithmetic;
import cern.jet.stat.Gamma;

public class Distributions {

	public static double dbinom(long k,long n,double p) {
		return Arithmetic.binomial(n, k)*Math.pow(p, k)*Math.pow(1-p, n-k);
	}
	
	public static double dhypergeom(long k,long n,long N,long K) {
		return (Arithmetic.binomial(K, k)
			   *Arithmetic.binomial(N-K, n-k))
			   /Arithmetic.binomial(N, n);
	}
	
	
	public static double dgeometric(long i,double p) {
		return Math.pow(1-p, i-1)*p;
	}
	
	public static double dnegbinom(long i,long r,double p) {
		return Arithmetic.binomial(i-1, i-r)*Math.pow(1-p, r)*Math.pow(p, i-r);
	}
	
	public static double dpois(int i,double p) {
		return Math.pow(p, i)*Math.exp(-p)/Arithmetic.factorial(i);
	}
	
	public static double dnorm(double x,double mu,double sig) {
		return Math.exp(Math.pow(x-mu, 2)/Math.sqrt(2*sig*sig))/Math.sqrt(2*Math.PI*sig*sig);
	}
	
	public static double dnormGender(double h,double gender,double hm,double hf,double sig,double p) {
		return Math.pow(p,gender==1 ? 1 : 0)*Math.pow(1-p,gender==1 ? 0 : 1)*dnorm(h,hm + (gender == 1 ? hf-hm : 0),sig);
	}
	
	public static double dchisq(double x,double k) {
		return (Math.pow(x,k/2.0 - 1.0)*Math.exp(-x/2.0))/(Math.pow(2,k/2.0)*Gamma.gamma(k/2.0));
	}
	
	public static double dexp(double x,double p) {
		return p*Math.exp(-x*p);
	}
	
	public static double dgamma(double x,double k,double p) {
		return Math.pow(p,k)*Math.pow(x,k-1)*Math.exp(-p*x)/Gamma.gamma(k);
	}
}
