package wiley.streaming.flume;

import java.nio.charset.Charset;

import org.apache.flume.Channel;
import org.apache.flume.Context;
import org.apache.flume.Event;
import org.apache.flume.EventDeliveryException;
import org.apache.flume.Transaction;
import org.apache.flume.conf.Configurable;
import org.apache.flume.sink.AbstractSink;

import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;

public class RedisSink extends AbstractSink implements Configurable {
	String           redisHost;
	int              redisPort;
	String           redisChannel;
	JedisPool        pool;
	Jedis            jedis;
	
	
	public void configure(Context context) {
		redisHost    = context.getString("redis-host", "localhost");
		redisPort    = context.getInteger("redis-port", 6379);
		redisChannel = context.getString("redis-channel", "flume"); 		
	}

	@Override
	public synchronized void start() {
		super.start();
		pool  = new JedisPool(new JedisPoolConfig(),redisHost,redisPort);
		jedis = pool.getResource();
	}

	@Override
	public synchronized void stop() {
		pool.returnResource(jedis);
		pool.destroy();
		super.stop();
	}

	
	
	public Status process() throws EventDeliveryException {
		Channel channel = getChannel();
		Transaction txn = channel.getTransaction();
		Status status = Status.READY;
		try {
			txn.begin();
			Event event = channel.take();
			if(jedis.publish(redisChannel, new String(event.getBody(),Charset.defaultCharset())) > 0) {
				txn.commit();
			}
		} catch(Exception e) {
			txn.rollback();
			status = status.BACKOFF;
		} finally {
			txn.close();
		}
		return status;
	}


	

}
