package wiley.streaming.nn;

import java.util.Random;

public class Layer {

	public static String printVec(double[] x) {
		StringBuilder sb = new StringBuilder();
		sb.append("{");
		for(int i=0;i<x.length;i++) { if(i > 0) sb.append(",");sb.append(x[i]); }
		sb.append("}");
		return sb.toString();
	}
	
	double[]   v;
	double[][] w;
	double[]   bW = null;

		Activation fn;
	
	
	public Layer(int units,int inputs,Activation fn,boolean bias) {
		this.fn = fn;
		v   = new double[units];

		w = new double[units][];
		for(int i=0;i<v.length;i++) w[i] = new double[inputs];
		if(bias)
			bW = new double[units];
		err = new double[units];		
	}
	
	public int units() { return v.length; }
	public double[] value()  { return v;   }
	
	
	double[]   err;
	double     E;	
	public double[] errors() { return err; }
	
	public double[] feed(double[] x) {
		for(int i=0;i<v.length;i++) {
			double[] W = w[i];
			v[i] = bW != null ? bW[i] : 0.0;
			for(int j=0;j<W.length;j++) v[i] += W[j]*x[j];
			v[i] = fn.f(v[i]);
		}
		return v;
	}
	
	public double[] backprop(double[] s) {
		double[] out = new double[w[0].length]; 
		E = 0;
		for(int i=0;i<v.length;i++) {
			err[i] = fn.df(v[i])*s[i];
			E += err[i]*err[i];
			//Compute the contribution of this node
			double[] W = w[i];
			for(int j=0;j<W.length;j++) out[j] += W[j]*err[i];
		}
		return out;
	}
	
	public double error() { return E; }
	
	public double[] update(double[] o,double r) {
		for(int i=0;i<v.length;i++) {
			if(bW != null)
				bW[i] += r*err[i];
			
			double[] W = w[i];
			for(int j=0;j<W.length;j++)
				W[j] += r*err[i]*o[j];
		}
		return v;
	}
	
	
	public void initialize(Random rng) {
		for(int i=0;i<v.length;i++) {
			for(int j=0;j<w[i].length;j++) 
				w[i][j] = 2*rng.nextDouble() - 1;
			bW[i] = 2*rng.nextDouble() - 1;
		}
		
	}
	
	public void initialize() {
		initialize(new Random());
	}
	
	
}
