package wiley.streaming.nn;

import java.util.ArrayList;

public class NeuralNetwork {

	int inputUnits = 0;
	ArrayList<Layer> layers = new ArrayList<Layer>();
	double learningRate     = 0.2;
	
	public NeuralNetwork learningRate(double l) {
		learningRate = l;
		return this;
	}

	public NeuralNetwork inputs(int inputUnits) {
		this.inputUnits = inputUnits;
		return this;
	}
	
	public NeuralNetwork momentumLayer(int units,Activation fn,boolean bias) {
		int inputs = (layers.size() == 0) ? this.inputUnits : layers.get(layers.size()-1).units();
		layers.add(new MomentumLayer(units,inputs,fn,bias));
		return this;		
	}
	
	public NeuralNetwork momentumLayer(int units,Activation fn) {
		return momentumLayer(units,fn,true);
	}
	
	public NeuralNetwork momentumLayer(int units) {
		return momentumLayer(units,Activation.tanh());
	}
	

	public NeuralNetwork layer(int units,Activation fn,boolean bias) {
		int inputs = (layers.size() == 0) ? this.inputUnits : layers.get(layers.size()-1).units();
		layers.add(new Layer(units,inputs,fn,bias));
		return this;
	}
	
	public NeuralNetwork layer(int units,Activation fn) {
		return layer(units,fn,true);
	}
	
	public NeuralNetwork layer(int units) {
		return layer(units,Activation.tanh());
	}
	
	public double[] feed(double[] x) {
		for(Layer l : layers) 
			x = l.feed(x);
		return x;
	}
	
	public double[] output() { return layers.get(layers.size()-1).value(); } 
	public double error() { 
		double E = 0;
		for(Layer l : layers) E += l.error();
		return E;
	}
	
	public NeuralNetwork train(double[] x,double[] y) {
		//First, feed this data through the network
		double[] out = feed(x);
		//Next compute the error between the output and the expected output
		double[] s = new double[out.length];
		for(int i=0;i<y.length;i++) s[i] = y[i] - out[i];
//		System.out.println(Layer.printVec(y)+" - "+Layer.printVec(out)+" => "+Layer.printVec(s));
		//Propagate this error through the values in reverse order
		for(int i=layers.size()-1;i>=0;i--) s = layers.get(i).backprop(s);
		for(Layer l : layers) x = l.update(x, learningRate);
		
		return this;
	}

	
	public NeuralNetwork initialize() {
		for(Layer l : layers) l.initialize();
		return this;
	}
	
	public static NeuralNetwork build() {
		return new NeuralNetwork();
	}
	
}
