package wiley.streaming.outlier;

import java.util.LinkedList;

public class ThresholdDetector {
	Forecaster f;

	int        n;
	double     sigma;
	LinkedList<Double> values = new LinkedList<Double>();
	double     s2;
	
	public ThresholdDetector(Forecaster f,int n,double sigma) {
		this.f = f;
		this.n     = n;
		this.sigma = sigma;
	}
	
	public ThresholdDetector(Forecaster f,int n) {
		this(f,n,3);
	}
	
	public ThresholdDetector(Forecaster f) {
		this(f,30);
	}
	
	public boolean observe(double y) {
		double err = y - f.forecast(y);
		double sig = Math.sqrt(s2/((double)n-1.0));
		//If this is an outlier don't include it in s2
		if(Math.abs(err)/sig > sigma)
			return true;
		//Otherwise update our standard deviation
		s2 += err*err;
		if(values.size() == n) 
			s2 -= values.removeFirst();
		values.add(err*err);
		
		return false;
	}
	
}
