package wiley.streaming.ts;

public class HoltWinters {
	public static enum Method {
		ADDITIVE,
		MULTIPLICATIVE
	};
	
	Method method = Method.ADDITIVE;
	double m = 0.0,b = 0.0;
	double a0,a1,a2;
	
	int seasons,cNdx,initialValue = 0;
	double[]  c;

	public HoltWinters(double a0,double a1,Method method,double a2,int seasons) {
		this.method = method;
		this.a0     = a0;
		this.a1     = a1;
		this.a2     = a2;
		this.seasons = seasons;
		this.c       = new double[seasons];
		this.initialValue = 2*seasons;
	}
	
	public HoltWinters level(double m) {
		this.m = m;
		return this;
	}
	public double level() { return m; }
	
	public HoltWinters slope(double b) {
		this.b = b;
		return this;
	}
	public double slope() { return b; }
	
	public void updateTrend(double y) {
		double m0 = m;
		m = a0*y + (1-a0)*(m0-b);
		b = a1*(m-m0) + (1-a1)*b;
	}
	
	public void update(double y) {
		int n     = (cNdx + 1) % c.length;
		double c_ = c[n];
		
		
		if(method == Method.ADDITIVE) {
			updateTrend(y-c_);
			c[cNdx] = a2*(y-m) + (1-a2)*c_;
		} else if(method == Method.MULTIPLICATIVE) {
			updateTrend(y/c_);
			c[cNdx] = a2*(y/m) + (1-a2)*c_;
		}
		cNdx = (cNdx + 1) % c.length;
	}
	
	public double forecast(double y) {
		if(initialValue > 0) {
			if(--initialValue == 0) {
				m /= (double)seasons;
			}
			return Double.NaN;
		} else {
			update(y);
			if(method == Method.ADDITIVE) return m + b + c[(cNdx + 1) % c.length];
			if(method == Method.MULTIPLICATIVE) return (m+b)*c[(cNdx + 1) % c.length];
			return Double.NaN;
		}
	}
	
	
	
}
