package wiley.aggregation.client;

import redis.clients.jedis.Jedis;
import wiley.aggregation.Aggregator;
import wiley.aggregation.Command;
import wiley.aggregation.Expirer;
import wiley.aggregation.Notifier;

public class RedisClient implements Expirer, Notifier {

	Jedis jedis;
	public RedisClient jedis(Jedis jedis) {
		this.jedis = jedis;
		return this;
	}
	
	Aggregator aggregator;
	public RedisClient aggregator(Aggregator aggregator) {
		this.aggregator = aggregator;
		aggregator.expirer(this);
		return this;
	}

	public void expire(String key, long retainMillis) {
		jedis.expire(key, (int)(retainMillis/1000));
	}

	public void append(long timestamp,String key,String value) {
		aggregator.each(timestamp, key, value, new Command<String>() {

			public String execute(long timestamp, long quantized,
					String quantizedString, String key, String value) {
				String k = key+":"+quantizedString;
				jedis.append(key, value);
				return k;
			}
			
		});
	}
	
	public void hincrBy(long timestamp,String key,String field,final long by) {
		aggregator.each(timestamp, key, field, new Command<String>() {

			public String execute(long timestamp, long quantized,
					String quantizedString, String key, String value) {
				String k = key+":"+quantizedString;
				jedis.hincrBy(key, value, by);
				return k;
			}
			
		});
	}
	
	public void incrBy(long timestamp,String key,long value) {
		aggregator.each(timestamp, key, value,new Command<Long>() {

			public String execute(long timestamp, long quantized,
					String quantizedString, String key, Long value) {
				String k = key+":"+quantizedString;
				jedis.incrBy(k, value);
				return k;
			}
			
		});
	}
	
	public void sadd(long timestamp,String key,String member) {
		aggregator.each(timestamp, key, member, new Command<String>() {

			public String execute(long timestamp, long quantized,
					String quantizedString, String key, String value) {
				String k = key+":"+quantizedString;
				jedis.sadd(k, value);
				return k;
			}
			
		});
	}
	
	
	public void zincrby(long timestamp,String key,final double score,String member) {
		aggregator.each(timestamp, key, member, new Command<String>() {

			public String execute(long timestamp, long quantized,
					String quantizedString, String key, String value) {
				String k = key+":"+quantizedString;
				jedis.zincrby(k, score, value);
				return k;
			}
			
		});
	}

	public void notify(String key,String resolution, long timestamp, long quantized,
			String quantizedString) {
		jedis.publish(key+":"+resolution, quantizedString);
	}
	
}
