package wiley.realtime;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.BitSet;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import wiley.realtime.hash.SerializableHasher;

public class BloomSet<E extends Serializable> implements Set<E> {

	BitSet               bits;
	int                  m,x;
	SerializableHasher[] hashes;
	ObjectOutputStream[] outputs;
	
	protected void initialize(int m,int[] seeds) throws IOException {
		this.m = m;
		bits   = new BitSet(m);
		hashes = new SerializableHasher[seeds.length];
		outputs= new ObjectOutputStream[seeds.length];
		for(int i=0;i<seeds.length;i++) {
			hashes[i]  = (new SerializableHasher()).seed(seeds[i]);
			outputs[i] = new ObjectOutputStream(hashes[i]);
		}
		bits.clear();		
	}
	
	public BloomSet(int m,int[] seeds) throws IOException {
		initialize(m,seeds);
	}
	
	public boolean contains(Object arg0) {
		for(int i=0;i<outputs.length;i++) {
			try {
				hashes[i].reset();
				outputs[i].writeObject(arg0);
				outputs[i].flush();
				if(!bits.get(hashes[i].hash() % m)) return false;
			} catch(IOException e) { }
		}
		return true;
	}
	
	public boolean add(E arg0) {
		if(!contains(arg0)) {
			for(int i=0;i<outputs.length;i++) {
				hashes[i].reset();
				try {
					outputs[i].writeObject(arg0);
					outputs[i].flush();
					int h = hashes[i].hash();
					x     = bits.get(h) ? 0 : 1;
					bits.set(h);
				} catch(IOException e) {
				}
			}
			return true;
		}
		return false;
	}

	public boolean addAll(Collection<? extends E> arg0) {
		boolean allNew = true;
		for(E e : arg0) allNew &= add(e);
		return allNew;
	}

	public void clear() {
		bits.clear();
	}

	
	public boolean containsAll(Collection<?> arg0) {
		for(Object e : arg0) if(!contains(e)) return false;
		return true;
	}

	public boolean isEmpty() {
		// TODO Auto-generated method stub
		return false;
	}

	public Iterator<E> iterator() {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean remove(Object arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean removeAll(Collection<?> arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean retainAll(Collection<?> arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	public int size() {
		int X = bits.cardinality();
		return (int) (Math.log(1 - (double)X/(double)bits.length())/(Math.log(1.0 - 1.0/(double)bits.length())*(double)hashes.length));
	}

	public Object[] toArray() {
		// TODO Auto-generated method stub
		return null;
	}

	public <T> T[] toArray(T[] arg0) {
		// TODO Auto-generated method stub
		return null;
	}

}
