package wiley.realtime;

import java.io.IOException;
import java.util.ArrayList;

import wiley.realtime.util.DyadicInterval;

public class CountMinRange {
	ArrayList<CountMinSketch<Integer>> sketches = new ArrayList<CountMinSketch<Integer>>();
	long N = 0;
	int  min = Integer.MAX_VALUE;
	int  max = Integer.MIN_VALUE;
	
	public CountMinRange(int width,int[] seeds) throws IOException {
		for(int i=0;i<32;i++) sketches.add(new CountMinSketch<Integer>(width,seeds));
	}
	
	public void increment(int value,long n) {
		if(value < min) min = value;
		if(value > max) max = value;
		for(int i=0;i<32;i++) {
			sketches.get(i).increment(value/(1 << i),n);
		}
		
		N += n;
	}
	
	public long range(int a,int b) {
		long count = 0;
		for(DyadicInterval d : DyadicInterval.createIntervals(a, b)) {
			count += sketches.get(d.level).get(d.start);
		}
		return count;
	}
	
	public double frequency(int a,int b) {
		return range(a,b)/N;
	}
	
	public int quantile(double q){
		if(q == 0.0) return min;
		if(q == 1.0) return max;
		
		int lo = min;
		int hi = max;
		while(lo <= hi) {
			int mid = lo + (hi - lo)/2;
			double val = frequency(min,mid);
			if(val < q) hi = mid - 1;
			else if(val > q) lo = mid + 1;
			else return mid;
		}
		return lo; //Return the nearest value
	}
}
