package wiley.storm;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import backtype.storm.Config;
import backtype.storm.Constants;
import backtype.storm.task.OutputCollector;
import backtype.storm.task.TopologyContext;
import backtype.storm.topology.IRichBolt;
import backtype.storm.topology.OutputFieldsDeclarer;
import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Tuple;
import backtype.storm.tuple.Values;

public class EventCounterBolt implements IRichBolt {

	
	
	
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 5081960478161021270L;

	
	int                     updates= 10;
	
	public EventCounterBolt updateSeconds(int updates) { this.updates = updates;return this; }
	public int updateSeconds() { return updates; }
	
	
	transient HashMap<String,Integer> counts;
	transient OutputCollector      collector;
	
	public void prepare(Map stormConf, TopologyContext context,
			OutputCollector collector) {
		this.counts    = new HashMap<String,Integer>();
		this.collector = collector;
	}

	public void declareOutputFields(OutputFieldsDeclarer declarer) {
		declarer.declare(new Fields("timestamp","key","count"));
	}

	public Map<String, Object> getComponentConfiguration() {
		Config conf = new Config();
		conf.put(Config.TOPOLOGY_TICK_TUPLE_FREQ_SECS, updates);
		return conf;
	}
	
	public static boolean isTick(Tuple tuple) {
		return Constants.SYSTEM_COMPONENT_ID.equals(tuple.getSourceComponent())
				&& Constants.SYSTEM_TICK_STREAM_ID.equals(tuple.getSourceStreamId());
	}
	
	public void execute(Tuple input) {
		if(isTick(input)) {
			for(Entry<String, Integer> e : counts.entrySet()) {
				collector.emit(new Values(System.currentTimeMillis(),e.getKey(),e.getValue()));
			}
			counts.clear();
		} else {
			String  key    = input.getString(0);
			Integer value  = counts.get(key);
			counts.put(key, 1 + (value == null ? 0 : value));
		}
		collector.ack(input);
	}

	public void cleanup() {

	}


}
