package wiley.storm.trident;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.log4j.Logger;

import kafka.consumer.Consumer;
import kafka.consumer.ConsumerConfig;
import kafka.consumer.ConsumerIterator;
import kafka.consumer.KafkaStream;
import kafka.javaapi.consumer.ConsumerConnector;
import kafka.message.MessageAndMetadata;


import backtype.storm.spout.SpoutOutputCollector;
import backtype.storm.task.TopologyContext;
import backtype.storm.topology.IRichSpout;
import backtype.storm.topology.OutputFieldsDeclarer;
import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Values;

public class SimpleKafkaSpout implements IRichSpout {
	private static final Logger LOG = Logger.getLogger(SimpleKafkaSpout.class);

	public static SimpleKafkaSpout spout() { return new SimpleKafkaSpout(); }
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 7267298313585421551L;

	protected String topic;
	public String topic() { return topic; }
	public SimpleKafkaSpout topic(String topic) { this.topic = topic;return this; }

	protected Properties props = new Properties();
	public SimpleKafkaSpout property(Object key,Object value) {
		props.put(key, value);
		return this;
	}
	
	
	public SimpleKafkaSpout configure(String[] args) {
		for(String arg : args) {
			String[] p = arg.split("=");
			if(p.length == 2) {
				if("topic".equalsIgnoreCase(p[0])) 
					topic = p[1];
				else
					property(p[0],p[1]);
			}
		}
		return this;
	}
	
	transient TopologyContext      context;
	transient SpoutOutputCollector collector;
	transient ConsumerConnector    consumer;
	transient Object[]             iterators;
	
	protected void checkDefault(Object key,Object value) {
		if(props.get(key) == null) props.put(key, value);
	}
	
	public void open(Map conf, TopologyContext context,
			SpoutOutputCollector collector) {
		this.context   = context;
		this.collector = collector;
		
		//Check and set defaults
		checkDefault("group.id","kafka-spout");
		checkDefault("zookeeper.connect","localhost");
		checkDefault("zookeeper.session.timeout.ms","400");
		checkDefault("zookeeper.sync.time.ms","200");
		checkDefault("auto.commit.interval.ms","1000");
		
		
		ConsumerConfig config = new ConsumerConfig(props);
		consumer = Consumer.createJavaConsumerConnector(config);
		HashMap<String,Integer> topicMap = new HashMap<String,Integer>();
		topicMap.put(topic, 1);
		List<KafkaStream<byte[],byte[]>> streams = consumer.createMessageStreams(topicMap).get(topic);
		iterators = new Object[streams.size()];
		for(int i=0;i<iterators.length;i++) {
			iterators[i] = streams.get(i).iterator();
		}
		
	}

	public void close() {
	}

	public void activate() {
	}

	public void deactivate() {
	}

	public void nextTuple() {
		for(Object i : iterators) {
			@SuppressWarnings("unchecked")
			ConsumerIterator<byte[],byte[]> iter = (ConsumerIterator<byte[],byte[]>)i;
			
			if(iter.hasNext()) {
				MessageAndMetadata<byte[],byte[]> msg = iter.next();
				//LOG.info("Got message from "+msg.partition()+" at offset "+msg.offset());
				String key   = msg.key() == null ? "" : new String(msg.key());
				String value = msg.message() == null ? "" : new String(msg.message());
				
				//LOG.info("KEY: "+key);
				//LOG.info("VALUE: "+value);
				
				try {
				collector.emit(new Values(
						msg.partition(),
						msg.offset(),
						key,
						value
							));
				} catch(Exception e) {
					collector.reportError(e);
				}
			}
			
		}
	}

	public void ack(Object msgId) {
	}

	public void fail(Object msgId) {
	}

	public void declareOutputFields(OutputFieldsDeclarer declarer) {
		declarer.declare(new Fields("partition","offset","key","value"));
	}

	public Map<String, Object> getComponentConfiguration() {
		return null;
	}

}
