package wiley.storm.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

import backtype.storm.tuple.Values;

import storm.trident.operation.BaseFunction;
import storm.trident.operation.TridentCollector;
import storm.trident.tuple.TridentTuple;

public class JSONToTuple extends BaseFunction {
	private static final Logger LOG = Logger.getLogger(JSONToTuple.class);

	/**
	 * 
	 */
	private static final long serialVersionUID = -4807527656688989687L;

	HashMap<String,Class<?>> types  = new HashMap<String,Class<?>>();
	ArrayList<String>        fields = new ArrayList<String>();
	transient JSONParser     parser = null;
	
	
	public JSONToTuple field(String name,Class<?> type) {
		fields.add(name);
		types.put(name, type);
		return this;
	}
	public List<String>         fields() { return fields; }
	public Map<String,Class<?>> types()  { return types;  }
	
	
	public void execute(TridentTuple tuple, TridentCollector collector) {
		if(parser == null) {
			parser = new JSONParser();
		}
		
		try {
			String line = tuple.getString(0);
			if(line == null) {
				LOG.info("Got a null message");
				return;
			}
			Object obj = parser.parse(line);
			if(obj instanceof JSONObject) {
				JSONObject json = (JSONObject)obj;
				Values output = new Values();
				for(String field : fields) {
					Object result = json.get(field);
//					LOG.info(field+"="+result);
					output.add(result);
				}
				collector.emit(output);
			} else LOG.info("Unable to handle: "+line);
		} catch (ParseException e) {
			collector.reportError(e);
		}
	}

	public static JSONToTuple fn() { return new JSONToTuple(); }
	
}
